/* bzflag
 * Copyright (c) 1993 - 2003 Tim Riker
 *
 * This package is free software;  you can redistribute it and/or
 * modify it under the terms of the license found in the file
 * named LICENSE that should have accompanied this file.
 *
 * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/* OSFile:
 *	Encapsulates FileIO and DIR IO 
 *	01/16/03 Inital Implementaion: Jeff Myers
 */

#ifndef _OSFILE_H_
#define _OSFILE_H_

#ifdef WIN32
  #pragma warning( disable : 4786)  // Disable warning message
  #define WIN32_LEAN_AND_MEAN		// Exclude rarely-used stuff from Windows headers
  #include <windows.h>
  #include <io.h>
  #include <direct.h>
#else
  #include <sys/types.h>
  #include <sys/stat.h>
  #include <unistd.h>
  #include <dirent.h>
  #include <ctype.h>
#endif

#include <stdio.h>
#include <string>
#include <vector>

class OSFile
{
public:
	OSFile ();
	OSFile(const char *name);
	OSFile(const char *name, bool bRead, bool bText);
	~OSFile();

	bool open(const char *name, bool bRead = true, bool bText= false);
	bool open(bool bRead = true, bool bText = false);
	bool close(void);

	void stdName(const char *name);
	void osName(const char *name);

	FILE* getFile(void);

	const char* getStdName(void);
	const char* getOSName(void);

	const char* getExtension(void);

	bool isOpen(void);

protected:
	char			sStdName[512];
	char			sOSName[512];

	FILE			*pFP;
};


class OSDir
{
public:
	OSDir();
	OSDir( const char* dirName);
	~OSDir();

	void setStdDir(const char* dirName);
	void setOSDir(const char* dirName);

	void makeStdDir(const char* dirName);
	void makeOSDir(const char* dirName);

	bool getNextFile(OSFile &oFile, bool bRecursive);

	const char* getStdName(void);
	const char* getOSName(void);

protected:
	char			sBaseStdDir[512];
	char			sBaseOSDir[512];
	
	bool windowsAddFileStack(const char *szPathName, bool bRecursive);
	bool linuxAddFileStack( const char *szPathName, bool bRecursive);

	void*			pNameList;
	int				iNamePos;
};


#endif//_OSFILE_H_