/* bzflag
 * Copyright (c) 1993 - 2003 Tim Riker
 *
 * This package is free software;  you can redistribute it and/or
 * modify it under the terms of the license found in the file
 * named LICENSE that should have accompanied this file.
 *
 * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

#ifdef _WIN32
#pragma warning(4:4786)
#endif

#include "common.h"

#include "AnimMesh.h"

#include "OSFile.h"
// UT types

#define DEBUG_NORMALS	0

typedef struct
{
	unsigned long	iVert;
}trU3DVertexI;

typedef struct
{
	char	i,j,k;
}ShortVec;
#pragma  pack(1) 
typedef struct 
{
	unsigned short	numPolys;
	unsigned short	numVerts;

	unsigned short	unusedRot;
	unsigned short	unusedFrame;

	unsigned long	unusedNormX;
	unsigned long	unusedNormY;
	unsigned long	unusedNormZ;

	unsigned long	scale;

	unsigned long	unused[3];

	unsigned char	name[12];

}trU3DDataHeader;

typedef enum
{
	eNormal1Side = 0,
	eNormal2Side ,
	eTransparent2Side,
	eMasked2Side,
	eModBlend2Side,
	eTagTri = 8
}teUTJamesPolyType;

typedef struct
{
	unsigned short		verts[3];
	char			type;
	char			color;
	unsigned char		rawUVs[3][2];
	char			textureID;
	char			flags;
}trU3DTriangle;
#pragma  pack() 

typedef struct
{
	trU3DVertexI	*verts;
	trU3DVertexI	*shortNorms;
	FaceVertNorm	*vertNorms;
}trU3DFrame;

// unreal utils
void setShortVec (ShortVec &outVec,Vertex3D&inVec) 
{
	outVec.i = (char)(inVec.x * 127.0f);
	outVec.j = (char)(inVec.y * 127.0f);
	outVec.j = (char)(inVec.z * 127.0f);
}

void getShortVec (Vertex3D &outVec,ShortVec &inVec) 
{
	outVec.x = (float)(inVec.i * 0.0078740157480315f);
	outVec.y = (float)(inVec.j * 0.0078740157480315f);
	outVec.z = (float)(inVec.k * 0.0078740157480315f);
}

void normalizeShortVec (ShortVec	&vec) 
{	
	float	len = (float)sqrt((float)vec.i*vec.i+vec.j*vec.j+vec.k*vec.k);

	vec.i = (char)((float)vec.i/(len * 127.0f));
	vec.j = (char)((float)vec.j/(len * 127.0f));
	vec.k = (char)((float)vec.k/(len * 127.0f));
}

void getIntVert (unsigned int inVert, Vertex3D &outVert) 
{
	outVert.x = ((float)(inVert & 0x7ff)*0.125f);
	
	if (outVert.x > 128.0f)
		outVert.x -= 256.0f;

	outVert.y = ((float)((inVert>>11) & 0x7ff)*0.125f);

	if (outVert.y > 128.0f)
		outVert.y -= 256.0f;

	outVert.z = ((float)((inVert>>22) & 0x3ff)*0.25f);

	if (outVert.z > 128.0f)
		outVert.z -= 256.0f;
}

unsigned int makeIntVert (Vertex3D &inVert)
{
	return ((int)(inVert.x * 8.0)&0x7ff)|(((int)(inVert.y*8.0)&0x7ff)<<11)|(((int)(inVert.z*4.0)&0x3ff)<<22);
}

bool is2Sided (unsigned char cFlag)
{
	if (((teUTJamesPolyType)cFlag ==eNormal2Side) ||
		((teUTJamesPolyType)cFlag ==eTransparent2Side) ||
		((teUTJamesPolyType)cFlag ==eMasked2Side) ||
		((teUTJamesPolyType)cFlag ==eModBlend2Side))
		return true;

	return false;
}

// MD3 structs

typedef struct
{
	char				name[64];
	Vertex3D			pos;
	float				rawMatrix[9];
	int					id;
}tMD3Tag;

typedef struct
{
	char	str[68];
}tMD3String;

typedef struct
{
	char	id[4];
	tMD3String	name;
	long	numMeshFrames;
	long	numSkins;
	long	numVerts;
	long	numTris;
	long	triStart;
	long	headerSize;
	long	TXvertStart;
	long	vertStart;
	long	meshSize;
}tMD3MeshHeader;

typedef struct
{
	long		a,b,c;
	int			id;
}tMD3IndexedTri;

typedef struct
{
	Vertex3D	na,nb,nc;
}trMD3VertNormals;

typedef struct
{
	float u,v;
}tMD3uv;

typedef struct
{
	signed short	x,y,z;
	unsigned char	u,v;
}tMD3iVertex;

typedef struct
{
	char	id[4];
	long	vers;
	char	fileName[68];
	long	numBoneFrames;
	long	numTags;
	long	numMeshes;
	long	numSkins;
	long	headerLen;
	long	tagStart;
	long	tagEnd;
	long	fileSize;
}tMD3FileHeader;

typedef struct
{
	Vertex3D	mins;
	Vertex3D	maxs;
	Vertex3D	pos;
	float		scale;
	char		creator[16];
}tMD3BoneFrame;
// common utils

// vertex utils
void getInterpVert (Vertex3D &v1, Vertex3D &v2, Vertex3D &outVert, float param)
{
	outVert = v1 + ((v2-v1)*param);
}

void getInterpVec (Vertex3D &v1, Vertex3D &v2, Vertex3D &outVec, float param)
{
	outVec = v1 + ((v2-v1)*param);
	set_mag(outVec,1.0f);
}

// text utitls
bool getLine (char *pData, FILE *fp)
{
	char	*pPtr = pData;
	char	thisChar;

	*pPtr = NULL;

	if (fread(&thisChar,1,1,fp) == 0)
		return true;

	while ((thisChar != 13)) {
		*pPtr = thisChar;
		pPtr++;

		if (fread(&thisChar,1,1,fp) == 0) {
			*pPtr = NULL;
			return true;
		}
	}
	*pPtr = NULL;

	fread(&thisChar,1,1,fp); // get the 10

	return false;
}

bool getLine2(char *pData, FILE *fp)
{
	char	*pPtr = pData;
	char	thisChar;

	*pPtr = NULL;

	if (fread(&thisChar,1,1,fp) == 0)
		return true;

	while ((thisChar != '\n')) {
		*pPtr = thisChar;
		pPtr++;

		if (fread(&thisChar,1,1,fp) == 0) {
			*pPtr = NULL;
			return true;
		}
	}
	*pPtr = NULL;

	return false;
}

//*********************************Mesh***************************

Mesh::Mesh()
{
	rScale.x = rScale.y = rScale.z = 1.0f;
	pModel = NULL;
}

Mesh::~Mesh()
{

}

void Mesh::draw (int thisFrame, int nextFrame, float param)
{
	if (!pModel)
		return;

	if (pModel->getNumFrames() < 1)
		return;

	Vertex3D	rThisV,rNextV,rInterpV,rNormal;

	AnimFrame	*pThisFrame = pModel->getFrame(thisFrame);
	AnimFrame	*pNextFrame = pModel->getFrame(nextFrame);

	IndexedFaceList::iterator		faceItr = vFaces.begin();

	int face = 0;
	glBegin(GL_TRIANGLES);

	while (faceItr != vFaces.end()) {
		for (int i = 0; i < 3; i++) {
				glTexCoord2f(faceItr->verts[i].tc.s,faceItr->verts[i].tc.t);

				rThisV = vFaceNormalFrames[thisFrame][face].norms[i];
				rNextV =  vFaceNormalFrames[nextFrame][face].norms[i];
				getInterpVec(rThisV,rNextV,rNormal,param);

				glNormal3f(rNormal.x,rNormal.y,rNormal.z);

				rThisV = pThisFrame->vVerts[faceItr->verts[i].index];
				rNextV = pNextFrame->vVerts[faceItr->verts[i].index];
				getInterpVert(rThisV,rNextV,rInterpV,param);

				rInterpV *= rScale;
				
				glVertex3f(rInterpV.x,rInterpV.y,rInterpV.z);
		}
		face++;
		faceItr++;
	}
	glEnd();
}

void Mesh::index (void)
{

}

void Mesh::reverse (void)
{
	IndecedVert	rTemp;

	IndexedFaceList::iterator	faceItr = vFaces.begin();

	while (faceItr != vFaces.end()) {
		rTemp = faceItr->verts[0];
		faceItr->verts[0] = faceItr->verts[2];
		faceItr->verts[2] = rTemp;

		faceItr++;
	}

	FaceNormFrameList::iterator normFrameItr = vFaceNormalFrames.begin();

	while (normFrameItr != vFaceNormalFrames.end()) {
		FaceNormList::iterator normItr = normFrameItr->begin();
		while (normItr != normFrameItr->end()) {
			normItr->faceNorm *= -1;
			normItr++;
		}
		normFrameItr++;
	}
}


typedef std::vector<int>	tvIntList;
typedef std::map<int,tvIntList> tmIntListIntMap;

void Mesh::smooth (float fAngle)
{
	float fLimit = cos(fAngle);

	tmIntListIntMap	mvFaceMap;
	tvIntList				vFaceList;
	int	face = 0;

	AnimFrame	*pFrame = pModel->getFrame(0);

	IndexedFaceList::iterator	faceItr = vFaces.begin();
	IndexedFaceList::iterator otherFaceItr;

	while (faceItr != vFaces.end()) {
		for (int i = 0; i < 3; i++) {
			vFaceList.clear();
			int iVertIndex = faceItr->verts[i].index;

			int iOtherFace = 0;
			otherFaceItr = vFaces.begin();

			while (otherFaceItr != vFaces.end()) {
				if ((otherFaceItr->verts[0].index == iVertIndex) || (otherFaceItr->verts[1].index == iVertIndex) || (otherFaceItr->verts[2].index == iVertIndex))
					vFaceList.push_back(iOtherFace);

				iOtherFace++;
				otherFaceItr++;
			}
			mvFaceMap[iVertIndex] = vFaceList;
		}
		face++;
		faceItr++;
	}

	FaceNormFrameList::iterator	rFrameNormIr = vFaceNormalFrames.begin();
	tvIntList::iterator		faceListItr;

	Vertex3D		fThisNorm;
	Vertex3D		fAcumNorm;
	int				iTotalNorms;

	while (rFrameNormIr != vFaceNormalFrames.end()) {
		faceItr = vFaces.begin();
		face = 0;

		while (faceItr != vFaces.end()) {
			fThisNorm = (*rFrameNormIr)[face].faceNorm;
			for (int i = 0; i < 3; i++) {
				int iVertIndex = faceItr->verts[i].index;
				
				faceListItr = mvFaceMap[iVertIndex].begin();

				fAcumNorm = fThisNorm;
				iTotalNorms = 1;

				while (faceListItr != mvFaceMap[iVertIndex].end()) {
					if ((*faceListItr) != face) {
						if (dot(fThisNorm,(*rFrameNormIr)[(*faceListItr)].faceNorm)>fLimit) {
							fAcumNorm += (*rFrameNormIr)[(*faceListItr)].faceNorm;
							iTotalNorms ++;

						}
					}
					faceListItr++;
				}

				if (iTotalNorms > 1) {
					fAcumNorm /= (float)iTotalNorms;
					set_mag(fAcumNorm,1.0f);
				}
				(*rFrameNormIr)[face].norms[i] = fAcumNorm;
			}
			faceItr++;
			face++;
		}
		rFrameNormIr++;
	}
}


//*********************************CAnimModel***************************
AnimModel::AnimModel()
{
	rScale.x = rScale.y = rScale.z = 1.0f;
	rOffset.x = rOffset.y = rOffset.z = 0.0f;
	rRotation.x = rRotation.y = rRotation.z = 0.0f;
}

AnimModel::~AnimModel()
{

}
//*********************FILE IO*****************

bool AnimModel::loadA3D (const char * fileName)
{
	OSFile	oFile (fileName);

	oFile.open();
	FILE	*fp = oFile.getFile();
	if (!fp)
		return false;

	char	tagName[4] = {'A','3','D','1'};
	int		iValidate = *((int*)(tagName));
	int		iFileTag;
	fread(&iFileTag,4,1,fp);

	if (iFileTag != iValidate) {
		fclose(fp);
		return false;
	}

	// global object data
	fread(&rOffset,sizeof(Vertex3D),1,fp);
	fread(&rRotation,sizeof(Vertex3D),1,fp);
	fread(&rScale,sizeof(Vertex3D),1,fp);

	char	tempStr[256];

	// seq data
	vSequences.clear();

	int iSeqs;// = m_vSequences.size();
	fread(&iSeqs,sizeof(int),1,fp);

	Sequence	rSeq;

	for (int iSeq = 0; iSeq < iSeqs; iSeq++) {
		fread(&rSeq.id,sizeof(unsigned int),1,fp);
		
		fread(tempStr,256,1,fp);
		rSeq.name = tempStr;

		fread(tempStr,256,1,fp);
		rSeq.group = tempStr;

		fread(&(rSeq.start),sizeof(unsigned int),1,fp);
		fread(&(rSeq.len),sizeof(unsigned int),1,fp);
		fread(&(rSeq.speed),sizeof(float),1,fp);
		
		vSequences.push_back(rSeq);
	}

	// events data
	vEvents.clear();

	int iEvents;// = m_vEvents.size();
	fread(&iEvents,sizeof(int),1,fp);

	Event	rEvent;
	int		iEventSeq;

	for (int iEvent = 0; iEvent < iEvents; iEvent++) {
		fread(tempStr,256,1,fp);
		rEvent.name = tempStr;

		fread(&iEventSeq,sizeof(unsigned int),1,fp);
		rEvent.seq = &(vSequences[iEventSeq]);

		fread(&(rEvent.time),sizeof(float),1,fp);
		
		vEvents.push_back(rEvent);
	}

	// tag data
	vTags.clear();

	int iTags;// = m_vTags.size();
	fread(&iTags,sizeof(int),1,fp);

	Tag	rTag;

	for (int iTag = 0; iTag < iTags; iTag++) {
		fread(tempStr,256,1,fp);
		rTag.name = tempStr;

		fread(&(rTag.tagID),sizeof(int),1,fp);
		fread(&(rTag.tagmateID),sizeof(int),1,fp);
		
		vTags.push_back(rTag);
	}

	// frame data
	vFrames.clear();

	int iFrames;// = m_vFrames.size();
	fread(&iFrames,sizeof(int),1,fp);

	int iVerts;// = m_vFrames.size();
	fread(&iVerts,sizeof(int),1,fp);
	
	AnimFrame				rFrame;
	Vertex3D				rVertex;
	TagCS					rTagCS;

	for (int iFrame = 0; iFrame < iFrames; iFrame++) {
		rFrame.vVerts.clear();
		rFrame.vTagCSs.clear();

		for (int iVert = 0; iVert < iVerts; iVert++) {
			fread(&rVertex,sizeof(Vertex3D),1,fp);
			rFrame.vVerts.push_back(rVertex);
		}

		// then the TagCSList
		for (int iTag = 0; iTag < iTags; iTag++) {
			fread(&rTagCS,sizeof(TagCS),1,fp);
			rFrame.vTagCSs.push_back(rTagCS);
		}
		vFrames.push_back(rFrame);
	}

	// mesh data
	vMeshes.clear();

	int	iMeshes;// = m_vMeshes.size();
	fread(&iMeshes,sizeof(int),1,fp);

	Mesh				oMesh;
	IndexedFace		rFace;
	FaceNormList		vNormList;
	int					faces;
	FaceVertNorm		vFaceNorm;

	oMesh.pModel = this;

	for (int iMesh = 0; iMesh < iMeshes; iMesh++) {
		oMesh.vFaces.clear();
		oMesh.vFaceNormalFrames.clear();
		oMesh.rScale = rScale;

		fread(tempStr,256,1,fp);
		oMesh.sName = tempStr;

		faces = 0;
		fread(&faces,sizeof(int),1,fp);

		for (int face = 0; face < faces; face++) {
			fread(&(rFace.verts),sizeof(IndecedVert)*3,1,fp);
			oMesh.vFaces.push_back(rFace);
		}

		for (int iFrame = 0; iFrame < iFrames; iFrame++) {
			vNormList.clear();
			
			for (int face = 0; face < faces; face++) {	// one per face
				fread(&(vFaceNorm),sizeof(FaceVertNorm),1,fp);
				vNormList.push_back(vFaceNorm);
			}
			oMesh.vFaceNormalFrames.push_back(vNormList);
		}
		vMeshes.push_back(oMesh);
	}
	oFile.close();

	return true;
}

bool AnimModel::loadU3D (const char *path, const char * fileName)
{
	OSDir oDir (path);

	if (!fileName)
		return false;

	char	aName[255];
	char	dName[255];
	char	fileNameToOpen[255];
	char	fileNameName[255];

	short 				iFrames;
	unsigned short		iFrameSize;

	strcpy (fileNameName,oDir.getOSName());
	strcat (fileNameName,"\\");
	strcat (fileNameName,fileName);

	strcpy (fileNameToOpen,fileNameName);
	strcat (fileNameToOpen,".uc");

	// load UC file
	loadUCFile(fileNameToOpen,aName,dName);

	strcpy (fileNameToOpen,oDir.getOSName());
	strcat(fileNameToOpen,"\\");
	strcat(fileNameToOpen,dName);

	FILE	*fp = fopen (fileNameToOpen,"rb");

	int iMeshes = -1;

	bool		bGotWepTag = false;
	Tag		rTagInfo;
	TagCS		rTagCS;
	int			aTagIndecies[3];

	trU3DDataHeader	rDataHeader;
	trU3DTriangle	*pTris = NULL;
	// read in the tris
	if (fp) {
		fread(&rDataHeader,sizeof(trU3DDataHeader),1,fp);

		pTris = (trU3DTriangle*)malloc(sizeof(trU3DTriangle)*rDataHeader.numPolys);

		for (int p = 0; p < rDataHeader.numPolys; p++) {
			fread(&pTris[p],sizeof(trU3DTriangle),1,fp);

			if (pTris[p].textureID > iMeshes)
				iMeshes = pTris[p].textureID;
		}
		fclose (fp);
	}
	else
		return false;

	// IDs are 0 based, this gives us a count
	iMeshes = iMeshes+1;

	// make the real triangles
	// we make a mesh for each texture

	char	tempStr[512];
	Mesh	oMesh;

	oMesh.pModel = this;

	IndexedFace	rFace;
	IndecedVert	rIndexedVert;

	for (int iMesh = 0; iMesh < iMeshes; iMesh++) {
		sprintf(tempStr,"UTMesh-%d",iMesh);
		oMesh.sName = tempStr;
		oMesh.vFaces.clear();
		oMesh.vFaceNormalFrames.clear();

		for (int faces = 0; faces < rDataHeader.numPolys; faces++) {
			if (pTris[faces].flags != eTagTri) {
				if (pTris[faces].textureID == iMesh) {
					for (int i = 0; i < 3; i++) {
						rIndexedVert.index = pTris[faces].verts[i];

						rIndexedVert.tc.s = pTris[faces].rawUVs[i][0] * 0.00390625f;
						rIndexedVert.tc.t = 1.0f - pTris[faces].rawUVs[i][1] * 0.00390625f;

						rFace.verts[i] = rIndexedVert;
					}

					if (pTris[faces].type == eTagTri) {
						if (!bGotWepTag) {
							aTagIndecies[0] = pTris[faces].verts[0];
							aTagIndecies[1] = pTris[faces].verts[1];
							aTagIndecies[2] = pTris[faces].verts[2];

							rTagInfo.name = "UTWepTag";
							rTagInfo.tagID = 0;
							rTagInfo.tagmateID = -1;
							bGotWepTag = true;
						}
					} else {
						if (is2Sided(pTris[faces].type)) {
							oMesh.vFaces.push_back(rFace);

							rIndexedVert = rFace.verts[0];
							rFace.verts[0] = rFace.verts[2];
							rFace.verts[2] = rIndexedVert;
						}
					}

					oMesh.vFaces.push_back(rFace);
				}
			}
		}
		oMesh.rScale = rScale;
		vMeshes.push_back(oMesh);
	}

	free(pTris);

	strcpy (fileNameToOpen,oDir.getOSName());
	strcat(fileNameToOpen,"\\");
	strcat(fileNameToOpen,aName);

	unsigned int iIntVert;
	Vertex3D	rVert;
	AnimFrame	rFrame;

	fp = fopen (fileNameToOpen,"rb");

	if (fp) {
		fread(&iFrames,2,1,fp);
		fread(&iFrameSize,2,1,fp);

		for (int f = 0;f< iFrames; f++) {
			rFrame.vTagCSs.clear();
			rFrame.vVerts.clear();

			for (int v = 0;v< rDataHeader.numVerts; v++) {
				fread(&iIntVert,4,1,fp);

				getIntVert(iIntVert,rVert);
				rFrame.vVerts.push_back(rVert);
			}
			vFrames.push_back(rFrame);
		}
		fclose (fp);
	}
	else
		return false;

	// now for the post pass

	Vertex3D	v1,v2,norm,p0,p1,p2;
	FaceNormList	vNormList;
	FaceVertNorm	vFaceNorm;

	MeshList::iterator	iMeshItr = vMeshes.begin();
	IndexedFaceList::iterator	faceItr;

	FrameList::iterator	iFrameItr;

	while (iMeshItr != vMeshes.end()) {
		iMeshItr->vFaceNormalFrames.clear();

		iFrameItr = vFrames.begin();
		
		while (iFrameItr != vFrames.end()) {
			iFrameItr->vTagCSs.clear();

			vNormList.clear();
			faceItr = iMeshItr->vFaces.begin();

			while (faceItr != iMeshItr->vFaces.end()) {
				p0 = iFrameItr->vVerts[faceItr->verts[0].index];
				p1 = iFrameItr->vVerts[faceItr->verts[1].index];
				p2 = iFrameItr->vVerts[faceItr->verts[2].index];

				v1 = p1-p0;
				set_mag(v1,1.0f);

				v2 = p2-p0;
				set_mag(v2,1.0f);

				cross(v1,v2,norm);

				norm.x *= -1;
				set_mag(norm,1.0f);

				vFaceNorm.faceNorm = norm;
				vFaceNorm.norms[0] = norm;
				vFaceNorm.norms[1] = norm;
				vFaceNorm.norms[2] = norm;

				vNormList.push_back(vFaceNorm);
				faceItr++;
			}
			iMeshItr->vFaceNormalFrames.push_back(vNormList);

			// now get the tag info for the frame
			if (bGotWepTag) {
				p0 = iFrameItr->vVerts[aTagIndecies[0]];
				p1 = iFrameItr->vVerts[aTagIndecies[1]];
				p2 = iFrameItr->vVerts[aTagIndecies[2]];
				p0 *= rScale;

				rTagCS.o = p0;

				p1 *= rScale;
				rTagCS.v = p1 - p0;
				set_mag(rTagCS.v,1.0f);

				p2 *= rScale;
				rTagCS.d = p2 - p0;
				set_mag(rTagCS.d,1.0f);

				cross(rTagCS.v,rTagCS.d,rTagCS.h);
				set_mag(rTagCS.h,1.0f);

				cross(rTagCS.h,rTagCS.v,rTagCS.d);
				set_mag(rTagCS.d,1.0f);

				iFrameItr->vTagCSs.push_back(rTagCS);
			
			}
			iFrameItr++;
		}
		iMeshItr++;
	}

	return true;
}

bool AnimModel::loadMD3 (const char * fileName)
{	
	if (!fileName)
		return false;
	
	OSFile	oFile (fileName);

	oFile.open();
	FILE	*fp = oFile.getFile();

	if (!fp)
		return false;
	
	rScale.x = rScale.y = rScale.z = 0.01f;
	rOffset.x = rOffset.y = rOffset.z = 0;
	rRotation.x = rRotation.y = rRotation.z = 0;

	int	iThisItem = 0;

	// file header
	tMD3FileHeader	rFileHeader;

	fread(&rFileHeader.id,4,1,fp);
	fread(&rFileHeader.vers,4,1,fp);
	fread(rFileHeader.fileName,68,1,fp);
	fread(&rFileHeader.numBoneFrames,4,1,fp);
	fread(&rFileHeader.numTags,4,1,fp);
	fread(&rFileHeader.numMeshes,4,1,fp);
	fread(&rFileHeader.numSkins,4,1,fp);
	fread(&rFileHeader.headerLen,4,1,fp);
	fread(&rFileHeader.tagStart,4,1,fp);
	fread(&rFileHeader.tagEnd,4,1,fp);
	fread(&rFileHeader.fileSize,4,1,fp);

	// bone frames
	tMD3BoneFrame		rBoneFrame;
	
	std::vector<tMD3BoneFrame>	vBoneList;

	for (iThisItem = 0; iThisItem < rFileHeader.numBoneFrames; iThisItem++) {
		fread(&rBoneFrame.mins,4*3,1,fp);
		fread(&rBoneFrame.maxs,4*3,1,fp);
		fread(&rBoneFrame.pos,4*3,1,fp);
		fread(&rBoneFrame.scale,4,1,fp);
		fread(rBoneFrame.creator,16,1,fp);
		vBoneList.push_back(rBoneFrame);
	}
		
	vFrames.clear();
	vTags.clear();

	AnimFrame	rFrame;
	// tags
	fseek(fp,rFileHeader.tagStart,SEEK_SET);

	Tag			rTag;
	TagCS			rTagCS;

	tMD3Tag			rMD3Tag;
	int				iTag = 0;
	for (iThisItem = 0; iThisItem < rFileHeader.numBoneFrames; iThisItem++) {
		rFrame.vTagCSs.clear();
		rFrame.vVerts.clear();
		for (iTag = 0; iTag < rFileHeader.numTags; iTag++) {
			fread(&rMD3Tag.name,64,1,fp);
			fread(&rMD3Tag.pos,3*4,1,fp);
			fread(&rMD3Tag.rawMatrix,9*4,1,fp);
			rMD3Tag.id = iTag;

			if (iThisItem == 0) {
				rTag.name = rMD3Tag.name;
				rTag.tagID = iTag;
				rTag.tagmateID = false;

				vTags.push_back(rTag);
			}

			rTagCS.o= rMD3Tag.pos;

			rTagCS.h.x = rMD3Tag.rawMatrix[0];
			rTagCS.h.y = rMD3Tag.rawMatrix[1];
			rTagCS.h.z = rMD3Tag.rawMatrix[2];

			rTagCS.v.x = rMD3Tag.rawMatrix[3];
			rTagCS.v.y = rMD3Tag.rawMatrix[4];
			rTagCS.v.z = rMD3Tag.rawMatrix[5];

			rTagCS.d.x = rMD3Tag.rawMatrix[6];
			rTagCS.d.y = rMD3Tag.rawMatrix[7];
			rTagCS.d.z = rMD3Tag.rawMatrix[8];

			rFrame.vTagCSs.push_back(rTagCS);
		}
		vFrames.push_back(rFrame);
	}
	fseek(fp,rFileHeader.tagEnd,SEEK_SET);

	int				iThisMeshMark = rFileHeader.tagEnd;

	tMD3MeshHeader	rMeshHeader;
	int				iFrames;
	int				iTemp;
	tMD3String		rString;
	tMD3IndexedTri	rTri;
	tMD3uv			rUV;
	tMD3iVertex		rVert;

	Vertex3D		rVertex;

	Mesh			oMesh;

	IndexedFace				rFace;
	std::vector<std::string>	vSkinList;
	std::string					str;

	std::vector<tMD3IndexedTri>	vTempTris;
	std::vector<tMD3uv>			vTempUVs;
	vMeshes.clear();

	oMesh.pModel = this;
	oMesh.rScale = rScale;

	int		iThisVert = 0;

	for (iThisItem = 0; iThisItem < rFileHeader.numMeshes; iThisItem++) {
		fread(&rMeshHeader.id,4,1,fp);
		fread(&rMeshHeader.name,68,1,fp);
		fread(&rMeshHeader.numMeshFrames,4,1,fp);
		fread(&rMeshHeader.numSkins,4,1,fp);
		fread(&rMeshHeader.numVerts,4,1,fp);
		fread(&rMeshHeader.numTris,4,1,fp);
		fread(&rMeshHeader.triStart,4,1,fp);
		fread(&rMeshHeader.headerSize,4,1,fp);
		fread(&rMeshHeader.TXvertStart,4,1,fp);
		fread(&rMeshHeader.vertStart,4,1,fp);
		fread(&rMeshHeader.meshSize,4,1,fp);
		
		oMesh.sName = rMeshHeader.name.str;
		oMesh.vFaceNormalFrames.clear();
		oMesh.vFaces.clear();

		vSkinList.clear();
		// skins
		fseek(fp,iThisMeshMark+rMeshHeader.headerSize,SEEK_SET);
		for (iTemp = 0; iTemp < rMeshHeader.numSkins; iTemp++) {
			fread(&rString.str,68,1,fp);
			str = rString.str;
			vSkinList.push_back(str);
		}
		// tris
		vTempTris.clear();

		fseek(fp,iThisMeshMark+rMeshHeader.triStart,SEEK_SET);
		for (iTemp = 0; iTemp<rMeshHeader.numTris; iTemp++) {
			fread(&rTri,4*3,1,fp);
			vTempTris.push_back(rTri);
		}

		// UVs
		fseek(fp,iThisMeshMark+rMeshHeader.TXvertStart,SEEK_SET);
		for (iTemp = 0; iTemp <rMeshHeader.numVerts; iTemp++) {
			fread(&rUV,4*2,1,fp);
			vTempUVs.push_back(rUV);
		}

		oMesh.vFaceNormalFrames.clear();
		oMesh.vFaces.clear();

		// make the real indexed 
		for (iTemp = 0; iTemp<rMeshHeader.numTris; iTemp++) {	
			rFace.verts[0].index = vTempTris[iTemp].c+iThisVert;
			rFace.verts[0].tc.s = vTempUVs[vTempTris[iTemp].c].u;
			rFace.verts[0].tc.t = vTempUVs[vTempTris[iTemp].c].v;

			rFace.verts[1].index = vTempTris[iTemp].b+iThisVert;
			rFace.verts[1].tc.s = vTempUVs[vTempTris[iTemp].b].u;
			rFace.verts[1].tc.t = vTempUVs[vTempTris[iTemp].b].v;

			rFace.verts[2].index = vTempTris[iTemp].a+iThisVert;
			rFace.verts[2].tc.s = vTempUVs[vTempTris[iTemp].a].u;
			rFace.verts[2].tc.t = vTempUVs[vTempTris[iTemp].a].v;

			oMesh.vFaces.push_back(rFace);
		}	
		
		// move pointer to Verts
		fseek(fp,iThisMeshMark+rMeshHeader.vertStart,SEEK_SET);

		for (iFrames = 0; iFrames <rMeshHeader.numMeshFrames; iFrames++) {
			for (iTemp = 0; iTemp < rMeshHeader.numVerts; iTemp++) {
				fread(&rVert.x,2,1,fp);
				fread(&rVert.y,2,1,fp);
				fread(&rVert.z,2,1,fp);

				fread(&rVert.u,1,1,fp);
				fread(&rVert.v,1,1,fp);
				
				rVertex.x = rVert.x;
				rVertex.y = rVert.y;
				rVertex.z = rVert.z;

				vFrames[iFrames].vVerts.push_back(rVertex);
			}
		}
		
		vMeshes.push_back(oMesh);

		iThisVert += rMeshHeader.numVerts;

		if (iThisItem < rFileHeader.numMeshes) {
			fseek(fp,iThisMeshMark+rMeshHeader.meshSize,SEEK_SET);
			iThisMeshMark += rMeshHeader.meshSize;
		}
	}
	oFile.close();

	IndexedFaceList::iterator	faceItr;
	MeshList::iterator		iMeshItr = vMeshes.begin();
	FrameList::iterator		iFrameItr = vFrames.begin();
	FaceNormList				vNormList;
	FaceVertNorm				vNorm;

	Vertex3D					p0,p1,p2,v1,v2,norm;

	while (iFrameItr != vFrames.end()) {
		iMeshItr = vMeshes.begin();
		while (iMeshItr != vMeshes.end()) {
			vNormList.clear();

			faceItr = iMeshItr->vFaces.begin();
			while (faceItr != iMeshItr->vFaces.end()) {
				p0 = iFrameItr->vVerts[faceItr->verts[0].index];
				p1 = iFrameItr->vVerts[faceItr->verts[1].index];
				p2 = iFrameItr->vVerts[faceItr->verts[2].index];

				v1 = p1-p0;
				v2 = p2-p0;
				
				cross(v1,v2,norm);
				norm.x *= -1;
				set_mag(norm,1.0f);

				vNorm.faceNorm = norm;
				vNorm.norms[0] = vNorm.norms[1] = vNorm.norms[2] = vNorm.faceNorm;
				
				vNormList.push_back(vNorm);
				faceItr++;
			}
			iMeshItr->vFaceNormalFrames.push_back(vNormList);
			iMeshItr++;
		}
		iFrameItr++;
	}
	return true;
}

bool AnimModel::loadOBJ (const char * fileName)
{
	if (!fileName)
		return false;
	
	OSFile	oFile (fileName);

//File.open(true,true);
	FILE	*fp = fopen(oFile.getOSName(),"rt");
	if (!fp)
		return false;

	vMeshes.clear();
	vTags.clear();
	vFrames.clear();

	rScale.x = rScale.y = rScale.z = 1.0f;
	rOffset.x = rOffset.y = rOffset.z = 0;
	rRotation.x = rRotation.y = rRotation.z = 0;

	char	line[1024];

	Vertex3D	rVert,p0,p1,p2,norm,v1,v2;
	Vertex2D	rTC;
	Vertex3D	rNorm;

	int			aTCIndecies[3];
	int			aNormIndecies[3];

	std::vector<Vertex3D>	vNormList;
	std::vector<Vertex2D>	vTCList;

	FaceVertNorm	rFaceNorm;
	IndexedFace	rFace;

	FaceNormList	rNormList;

	AnimFrame	rFrame;
	Mesh		oMesh;
	char		tempStr[128];

	oMesh.pModel = this;
	oMesh.rScale = rScale;

	line[0] = NULL;
	bool bDone = getLine2(line,fp);
	
	while (!bDone) {
		switch(line[0]) {
			case 'v':
			case 'V':
				switch(line[1]) {
					case 't':
					case 'T':
						sscanf(line,"%s %f %f",tempStr,&rTC.s,&rTC.t);
						vTCList.push_back(rTC);
					break;

					case 'n':
					case 'N':
						rNorm.x = rNorm.y = rNorm.z = 0;
						sscanf(line,"%s %f %f %f",tempStr,&rNorm.x,&rNorm.y,&rNorm.z);
						//rnorm *= -1;
						set_mag(rNorm,1.0f);
						vNormList.push_back(rNorm);
					break;
					
					default:
						sscanf(line,"%s %f %f %f",tempStr,&rVert.x,&rVert.y,&rVert.z);
						rFrame.vVerts.push_back(rVert);
					break;
				}
			break;

			case 'g':
			case 'G':
				if (oMesh.vFaces.size() < 0) {
					oMesh.vFaceNormalFrames.push_back(rNormList);
					vMeshes.push_back(oMesh);

					oMesh.vFaces.clear();
					rNormList.clear();
				}

				sscanf(line,"%s %s",tempStr,tempStr);
				oMesh.sName = tempStr;
			break;

			case 'f':
			case 'F':
				rFace.verts[0].index = rFace.verts[1].index = rFace.verts[2].index = 0;
				aTCIndecies[0] = aTCIndecies[1] = aTCIndecies[2] = 0;
				aNormIndecies[0] = aNormIndecies[1] = aNormIndecies[2] = 0;

				sscanf(line,"%s %d/%d/%d %d/%d/%d %d/%d/%d",tempStr,
					&rFace.verts[0].index,&aTCIndecies[0],&aNormIndecies[0],
					&rFace.verts[1].index,&aTCIndecies[1],&aNormIndecies[1],
					&rFace.verts[2].index,&aTCIndecies[2],&aNormIndecies[2]);

				rFace.verts[0].index--;
				rFace.verts[1].index--;
				rFace.verts[2].index--;

				rFace.verts[0].tc = vTCList[aTCIndecies[0]-1];
				rFace.verts[1].tc = vTCList[aTCIndecies[1]-1];
				rFace.verts[2].tc = vTCList[aTCIndecies[2]-1];

				rFaceNorm.norms[0] = vNormList[aNormIndecies[0]-1];
				rFaceNorm.norms[1] = vNormList[aNormIndecies[1]-1];
				rFaceNorm.norms[2] = vNormList[aNormIndecies[2]-1];


				p0 = rFrame.vVerts[rFace.verts[0].index];
				p1 = rFrame.vVerts[rFace.verts[1].index];
				p2 = rFrame.vVerts[rFace.verts[2].index];

				v1 = p1-p0;
				set_mag(v1,1.0f);

				v2 = p2-p0;
				set_mag(v2,1.0f);

				cross(v1,v2,norm);

				norm.x *= -1;
				set_mag(norm,1.0f);

				rFaceNorm.faceNorm = norm;

				rNormList.push_back(rFaceNorm);
				oMesh.vFaces.push_back(rFace);
			break;
		}
		line[0] = NULL;
		bDone = getLine2(line,fp);
	}

	vFrames.push_back(rFrame);
	oMesh.vFaceNormalFrames.push_back(rNormList);
	vMeshes.push_back(oMesh);

	fclose(fp);

	return true;
}

bool AnimModel::loadBZG (const char * fileName)
{
	return false;
}

bool AnimModel::saveA3D (const char * fileName)
{
	OSFile	oFile (fileName);

	oFile.open(false,false);
	FILE	*fp = oFile.getFile();
	if (!fp)
		return false;

	char	tagName[4] = {'A','3','D','1'};
	fwrite(tagName,4,1,fp);

	// global object data
	fwrite(&rOffset,sizeof(Vertex3D),1,fp);
	fwrite(&rRotation,sizeof(Vertex3D),1,fp);
	fwrite(&rScale,sizeof(Vertex3D),1,fp);

	// seq data
	int iSeqs = vSequences.size();
	fwrite(&iSeqs,sizeof(int),1,fp);

	SequenceListList::iterator seqItr = vSequences.begin();

	char	tempStr[256];

	while (seqItr != vSequences.end()) {
		fwrite(&(seqItr->id),sizeof(unsigned int),1,fp);
		
		strcpy(tempStr,seqItr->name.c_str());
		fwrite(tempStr,256,1,fp);

		strcpy(tempStr,seqItr->group.c_str());
		fwrite(tempStr,256,1,fp);

		fwrite(&(seqItr->start),sizeof(unsigned int),1,fp);
		fwrite(&(seqItr->len),sizeof(unsigned int),1,fp);
		fwrite(&(seqItr->speed),sizeof(float),1,fp);
		seqItr++;
	}

	// events data
	int iEvents = vEvents.size();
	fwrite(&iEvents,sizeof(int),1,fp);

	EventListList::iterator eventItr = vEvents.begin();

	while (eventItr != vEvents.end()) {
		strcpy(tempStr,eventItr->name.c_str());
		fwrite(tempStr,256,1,fp);

		fwrite(&(eventItr->seq->id),sizeof(unsigned int),1,fp);
		fwrite(&(eventItr->time),sizeof(float),1,fp);
		eventItr++;
	}

	// tag data
	int iTags = vTags.size();
	fwrite(&iTags,sizeof(int),1,fp);

	TagList::iterator tagItr = vTags.begin();

	while (tagItr != vTags.end()) {
		strcpy(tempStr,tagItr->name.c_str());
		fwrite(tempStr,256,1,fp);

		fwrite(&(tagItr->tagID),sizeof(int),1,fp);
		fwrite(&(tagItr->tagmateID),sizeof(int),1,fp);
		tagItr++;
	}

	// frame data
	int iFrames = vFrames.size();
	fwrite(&iFrames,sizeof(int),1,fp);

	int iVerts = vFrames[0].vVerts.size();
	fwrite(&iVerts,sizeof(int),1,fp);

	FrameList::iterator	frameItr = vFrames.begin();
	VertexList::iterator	vertItr;
	TagCSList::iterator	tagCSItr;

	while (frameItr != vFrames.end()) {
		vertItr = frameItr->vVerts.begin();
		while (vertItr != frameItr->vVerts.end()) {
			fwrite(&(*vertItr),sizeof(Vertex3D),1,fp);
			vertItr++;
		}
		// then the TagCSList

		tagCSItr = frameItr->vTagCSs.begin();

		while (tagCSItr != frameItr->vTagCSs.end()) {
			fwrite(&(*tagCSItr),sizeof(TagCS),1,fp);
			tagCSItr++;
		}
		frameItr++;
	}

	int	iMeshes = vMeshes.size();
	fwrite(&iMeshes,sizeof(int),1,fp);

	int	faces;

	MeshList::iterator				meshItr = vMeshes.begin();
	IndexedFaceList::iterator			faceItr;
	FaceNormFrameList::iterator		faceNormalItr;
	FaceNormList::iterator			normalItr;
	while (meshItr != vMeshes.end()) {
		strcpy(tempStr,meshItr->sName.c_str());
		fwrite(tempStr,256,1,fp);

		faces = meshItr->vFaces.size();
		fwrite(&faces,sizeof(int),1,fp);

		faceItr = meshItr->vFaces.begin();
		while (faceItr != meshItr->vFaces.end()) {
			fwrite(&(faceItr->verts),sizeof(IndecedVert)*3,1,fp);
			faceItr++;
		}

		faceNormalItr = meshItr->vFaceNormalFrames.begin();
		
		while (faceNormalItr != meshItr->vFaceNormalFrames.end()) {	// one per frame
		
			normalItr = faceNormalItr->begin();

			while (normalItr != faceNormalItr->begin()) {	// one per face
				fwrite(&(*normalItr),sizeof(FaceVertNorm),1,fp);
				normalItr++;
			}
			faceNormalItr++;
		}

		meshItr++;
	}
	oFile.close();

	return true;
}

bool AnimModel::saveBZG (const char * fileName)
{
	return false;
}

int AnimModel::getMeshCount (void)
{
	return vMeshes.size();
}

int AnimModel::getMeshID (const char *meshName)
{
	if (!meshName || (vMeshes.size()<1))
		return -1;

	std::string	str = meshName;
	MeshList::iterator	itr = vMeshes.begin();

	int i = 0;
	while (itr != vMeshes.end()) {
		if (meshName == itr->sName)
			return i;
		i++;
		itr++;
	}
	return -1;
}

Mesh* AnimModel::getMesh (int iMeshID)
{
	if ((iMeshID < 0) || (iMeshID >= (signed)vMeshes.size()))
		return NULL;

	return &(vMeshes[iMeshID]);
}

int AnimModel::getNumFrames (void)
{
	return vFrames.size();
}

AnimFrame* AnimModel::getFrame (int iFrame)
{
	if ((iFrame < 0) || (iFrame >= (signed)vFrames.size()))
		return NULL;

	return &(vFrames[iFrame]);
}

int AnimModel::getNumSequences (void)
{
	return 	vSequences.size();
}

int AnimModel::getSequenceID (const char *seqName)
{
	if (!seqName || (vSequences.size()<1))
		return -1;

	std::string	str = seqName;
	SequenceListList::iterator	itr = vSequences.begin();

	int i = 0;
	while (itr != vSequences.end()) {
		if (seqName == itr->name)
			return i;

		i++;
		itr++;
	}
	return -1;
}

Sequence* AnimModel::getSequence (int iSeqID)
{
	if ((iSeqID < 0) || (iSeqID >= (signed)vSequences.size()))
		return NULL;

	return &(vSequences[iSeqID]);
}

int AnimModel::getNumEvents (void)
{
	return 	vEvents.size();
}

int AnimModel::getEventID (const char *eventName)
{
	if (!eventName || (vEvents.size()<1))
		return -1;

	std::string	str = eventName;
	EventListList::iterator	itr = vEvents.begin();

	int i = 0;
	while (itr != vEvents.end()) {
		if (eventName == itr->name)
			return i;

		i++;
		itr++;
	}
	return -1;
}

Event* AnimModel::getEvent (int iEventID)
{
	if ((iEventID < 0) || (iEventID >= (signed)vEvents.size()))
		return NULL;

	return &(vEvents[iEventID]);
}

void AnimModel::draw (int thisFrame, int nextFrame, float param)
{
	if ((vFrames.size() < 1))
		return;

	glPushMatrix();

	glRotatef(-rRotation.x,1,0,0);
	glRotatef(rRotation.y,0,1,0);
	glRotatef(rRotation.z,0,0,1);

	Vertex3D		tempV = rOffset;

	tempV *= rScale;
	tempV *= -1.0f;

	glTranslatef(tempV.x,tempV.y,tempV.z);
	
	for (int iMesh = 0; iMesh < getMeshCount(); iMesh ++) {
		vMeshes[iMesh].draw(thisFrame,nextFrame,param);
	}
	glPopMatrix();
}


//**************************** ut converters
void AnimModel::loadUCFile (char *fileName, char* aName, char *dName)
{
	aName[0] = NULL;
	dName[0] = NULL;
	
	FILE	*ucFile = fopen(fileName,"rb");

	if (!ucFile)
		return;

	char	line[512];
	char	label[256],command[256];
	char	*pLinePtr,*pPos;
	char	temp[512];
	char	tempStr[512];

	int		iSeq = 0;

	while (!getLine(line,ucFile)) {
		if (line[0] && (line[0] !='/')) {// not blank or a commennt
			sscanf(line,"%s",label);
			pLinePtr = line + strlen(label)+1;

			if (strcasecmp(label,"class")==0) {	// class label
				//class SGirl extends TournamentPlayer;
				// skip it
			}
			else if (strcasecmp(label,"#exec")==0) {	// exec command
				sscanf(pLinePtr,"%s",temp);// mesh or meshmap block
				pLinePtr += strlen(temp)+1;

				sscanf(pLinePtr,"%s",command); // command name
				pLinePtr += strlen(command)+1;

				if (strcasecmp(command,"IMPORT")==0) {			// IMPORT command
					//MESH=SGirl ANIVFILE=MODELS\SGirl_a.3d DATAFILE=MODELS\SGirl_d.3d
					sscanf(pLinePtr,"MESH=%s ANIVFILE=MODELS\\%s DATAFILE=MODELS\\%s",temp,aName,dName);
				}
				else if (strcasecmp(command,"LODPARAMS")==0) {	// LODPARAMS command
					//MESH=SGirl HYSTERESIS=0.00 STRENGTH=1.00 MINVERTS=10.00 MORPH=0.30 ZDISP=0.00
					// just skip it
				}
				else if (strcasecmp(command,"ORIGIN")==0) {	// ORIGIN command
					//MESH=SGirl X=-30.00 Y=-115.00 Z=20.00 YAW=0.00 ROLL=64.00 PITCH=-64.00
					sscanf(pLinePtr,"MESH=%s X=%f Y=%f Z=%f YAW=%f ROLL=%f PITCH=%f",temp,
											&rOffset.x,&rOffset.y,&rOffset.z,
											&rRotation.z,&rRotation.y,&rRotation.x);

					rRotation.x *= 1.40625f;
					rRotation.z *= 1.40625f;
					rRotation.y *= 1.40625f;
				}
				else if (strcasecmp(command,"SEQUENCE")==0) {	// SEQUENCE command
					//MESH=SGirl SEQ=Taunt1    STARTFRAME=5 NUMFRAMES=15 RATE=20.00 GROUP=Gesture

					// new seq 
					Sequence	seq;

					tempStr[0] = NULL;
					pPos = strstr(pLinePtr,"SEQ=");
					if (pPos)
						sscanf(pPos,"SEQ=%s",tempStr);
						
					seq.name = tempStr;

					tempStr[0] = NULL;
					pPos = strstr(pLinePtr,"GROUP=");
					if (pPos)
						sscanf(pPos,"GROUP=%s",tempStr);
					seq.group = tempStr;

					seq.start = 0;
					pPos = strstr(pLinePtr,"STARTFRAME=");
					if (pPos)
						sscanf(pPos,"STARTFRAME=%d",&seq.start);

					seq.len = 1;
					pPos = strstr(pLinePtr,"NUMFRAMES=");
					if (pPos)
						sscanf(pPos,"NUMFRAMES=%d",&seq.len);

					seq.speed = 15.0f;
					pPos = strstr(pLinePtr,"RATE=");
					if (pPos)
						sscanf(pPos,"RATE=%f",&seq.speed);

					seq.id = iSeq ++;
					vSequences.push_back(seq);
				}
				else if (strcasecmp(command,"SCALE")==0) {		// SCALE command
					//MESHMAP=SGirl X=0.06 Y=0.06 Z=0.13
					sscanf(pLinePtr,"%s X=%f Y=%f Z=%f",temp,&rScale.x,&rScale.z,&rScale.y);

					rScale.y = rScale.y/2.0f;
				}
				else if (strcasecmp(command,"NOTIFY")==0) {	// NOTIFY command
					//MESH=SGirl SEQ=RunLg     TIME=0.25 FUNCTION=PlayFootStep
					Event	event;
					
					tempStr[0] = NULL;
					pPos = strstr(pLinePtr,"SEQ=");
					if (pPos)
						sscanf(pPos,"SEQ=%s",tempStr);

					event.name = tempStr;

					tempStr[0] = NULL;
					pPos = strstr(pLinePtr,"FUNCTION=");
					if (pPos)
						sscanf(pPos,"FUNCTION=%s",tempStr);

					event.time = 0.0f;
					pPos = strstr(pLinePtr,"TIME=");
					if (pPos)
						sscanf(pPos,"TIME=%f",&event.time);

					vEvents.push_back(event);
				}
			}
		}
	}

	fclose (ucFile);
}


